@@ -74,9 +74,7 @@ struct _DownloaderViewPrivate
 	NotifyNotification *notification;
 #endif
 
-	guint idle_unref : 1;
 	guint source_id;
-	guint notification_timeout;
 };
 
 enum
@@ -200,7 +198,7 @@ remove_download (WebKitDownload *download,
 	WebKitDownloadStatus status;
 
 	g_signal_handlers_disconnect_matched
-		(download, G_SIGNAL_MATCH_DATA ,
+		(download, G_SIGNAL_MATCH_DATA,
 		 0, 0, NULL, NULL, view);
 
 	status = webkit_download_get_status (download);
@@ -222,8 +220,6 @@ prepare_close_cb (EphyEmbedShell *shell,
 	/* hide window already */
 	gtk_widget_hide (priv->window);
 
-	priv->idle_unref = FALSE;
-
 	/* cancel pending downloads */
 	g_hash_table_foreach_remove (priv->downloads_hash,
 				     (GHRFunc) remove_download, view);
@@ -237,14 +233,12 @@ prepare_close_cb (EphyEmbedShell *shell,
 static void
 downloader_view_init (DownloaderView *dv)
 {
-	g_object_ref (embed_shell);
-
+	_ephy_embed_shell_track_object (embed_shell, G_OBJECT (dv));
 	dv->priv = EPHY_DOWNLOADER_VIEW_GET_PRIVATE (dv);
 
 	dv->priv->downloads_hash = g_hash_table_new_full
 		(g_direct_hash, g_direct_equal, NULL,
 		 (GDestroyNotify)gtk_tree_row_reference_free);
-	dv->priv->idle_unref = TRUE;
 
 	downloader_view_build_ui (dv);
 
@@ -259,10 +253,14 @@ downloader_view_finalize (GObject *object)
 {
 	DownloaderView *dv = EPHY_DOWNLOADER_VIEW (object);
 	DownloaderViewPrivate *priv = dv->priv;
-	gboolean idle_unref = dv->priv->idle_unref;
 
 	if (priv->status_icon != NULL)
 	{
+		/* FIXME: this should not be necessary (setting visible to
+		 * FALSE), but we have to work-around a GTK+/gnome-panel bug:
+		 *   http://bugzilla.gnome.org/show_bug.cgi?id=340110
+		 */
+		gtk_status_icon_set_visible (priv->status_icon, FALSE);
 		g_object_unref (priv->status_icon);
 		priv->status_icon = NULL;
 	}
@@ -273,23 +271,9 @@ downloader_view_finalize (GObject *object)
 		priv->source_id = 0;
 	}
 	
-	if (priv->notification_timeout != 0)
-	{
-		g_source_remove (priv->notification_timeout);
-	}
-
 	g_hash_table_destroy (dv->priv->downloads_hash);
 
 	G_OBJECT_CLASS (downloader_view_parent_class)->finalize (object);
-
-	if (idle_unref)
-	{
-		ephy_object_idle_unref (embed_shell);
-	}
-	else
-	{
-		g_object_unref (embed_shell);
-	}
 }
 
 DownloaderView *
@@ -376,13 +360,17 @@ static char *
 ephy_download_get_name (WebKitDownload *download)
 {
 	const char *target;
+	char *user_destination;
 	char *result;
 
 	target = webkit_download_get_destination_uri (download);
+	user_destination = g_object_get_data (G_OBJECT (download),
+					      "user-destination-uri");
 
-	if (target)
+	if (user_destination || target)
 	{
-		result = g_path_get_basename (target);
+		result = g_path_get_basename (user_destination ?
+					      user_destination : target);
 	}
 	else
 	{
@@ -420,13 +408,22 @@ do_open_downloaded_file (DownloaderView *dv, WebKitDownload *download, gboolean
 {
 	DownloaderViewPrivate *priv = dv->priv;
 	GdkDisplay *gdk_display;
-	const char *destination_uri = webkit_download_get_destination_uri (download);
-	GFile *downloaded_file = g_file_new_for_uri (destination_uri);
+	const char *destination_uri;
+	char *user_destination;
+	GFile *downloaded_file;
 
 	gdk_display = gtk_widget_get_display (priv->window);
+
+	destination_uri = webkit_download_get_destination_uri (download);
+	user_destination = g_object_get_data (G_OBJECT (download),
+					      "user-destination-uri");
+
+	downloaded_file = g_file_new_for_uri (user_destination ?
+					      user_destination : destination_uri);
 	if (open_location)
 	{
-		ephy_file_browse_to (downloaded_file, gdk_x11_display_get_user_time (gdk_display));
+		ephy_file_browse_to (downloaded_file,
+				     gdk_x11_display_get_user_time (gdk_display));
 	}
 	else
 	{
@@ -601,10 +598,12 @@ download_status_changed_cb (WebKitDownload *download, GParamSpec *pspec, Downloa
                 update_download_row (dv, download);
 }
 
-static void
+static gboolean
 download_error_cb (WebKitDownload *download, gint error_code, gint error_detail, const gchar *reason, DownloaderView *dv)
 {
 	update_download_row (dv, download);
+
+	return FALSE;
 }
 
 static gboolean
@@ -617,30 +616,23 @@ update_buttons_timeout_cb (DownloaderView *dv)
 }
 
 #ifdef HAVE_LIBNOTIFY
-static gboolean
-queue_show_notification (DownloaderView *dv)
-{
-	if (gtk_status_icon_is_embedded (dv->priv->status_icon))
-	{
-		notify_notification_show (dv->priv->notification, NULL);
-		dv->priv->notification_timeout = 0;
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static void
 show_notification_window (DownloaderView *dv)
 {
 	if (gtk_status_icon_is_embedded (dv->priv->status_icon))
-		notify_notification_show (dv->priv->notification, NULL);
+	{
+		notify_notification_attach_to_status_icon
+						(dv->priv->notification,
+						 dv->priv->status_icon);
+	}
 	else
 	{
-		if (dv->priv->notification_timeout != 0)
-			g_source_remove (dv->priv->notification_timeout);
-		dv->priv->notification_timeout = g_timeout_add_seconds (1, (GSourceFunc) queue_show_notification, dv);
+		notify_notification_attach_to_status_icon
+						(dv->priv->notification,
+						 NULL);
 	}
+
+	notify_notification_show (dv->priv->notification, NULL);
 }
 #endif
 
